@@ -174,7 +174,8 @@ typedef enum {	normal,		/* nil */
 		eUser,		/* user at previous host in rev route */
 		eVerbose,	/* verbose log filename for "mail -v" */
 		eVia,		/* received via <medium> */
-		eWith		/* received with <protocol> */
+		eWith,		/* received with <protocol> */
+		eEnvEnd		/* Last of the envelope entries "env-end" */
 } HeaderClass;
 
 struct headerinfo {
