@@ -32,8 +32,8 @@
 	}
 #define SIGNAL_HOLD(SIG) \
 	sigblock(sigmask(SIG))
-#define SIGNAL_RELEASE(SIG) \
-	sigblock((~(sigmask(SIG))) & sigblock(0))
+#define SIGNAL_RELEASE(OLDMASK) \
+	sigsetmask(OLDMASK)
 #define SIGNAL_IGNORE(SIG) \
 	SIGNAL_HANDLE(SIG,SIG_IGN)
 
@@ -41,6 +41,7 @@
 
 #if	defined(SA_NOCLDSTOP)||defined(SA_ONSTACK)||defined(SA_RESTART)
 	/* ================ POSIX.1 STUFF ================ */
+#ifdef SA_INTERRUPT
 #define SIGNAL_HANDLE(X,Y) \
 	{	struct sigaction act;		\
 		act.sa_handler = Y;		\
@@ -71,6 +72,40 @@
 	}
 #define SIGNAL_IGNORE(SIG) \
 	SIGNAL_HANDLE(SIG,SIG_IGN)
+
+#else /* No  SA_INTERRUPT */
+
+#define SIGNAL_HANDLE(X,Y) \
+	{	struct sigaction act;		\
+		act.sa_handler = Y;		\
+		act.sa_flags   = 0;		\
+		sigemptyset(&act.sa_mask);	\
+		sigaction (X, &act, NULL);	\
+	}
+#define SIGNAL_HANDLESAVE(X,Y,OLD) \
+	{	struct sigaction act, oact;	\
+		act.sa_handler = Y;		\
+		act.sa_flags   = 0;		\
+		sigemptyset(&act.sa_mask);	\
+		sigemptyset(&oact.sa_mask);	\
+		sigaction (X, &act, &oact);	\
+		OLD = oact.sa_handler;		\
+	}
+#define SIGNAL_HOLD(SIG) \
+	{	sigset_t	sigmask;	\
+		sigemptyset(&sigmask);		\
+		sigaddset(&sigmask,SIG);	\
+		sigprocmask(SIG_BLOCK,&sigmask,NULL);	\
+	}
+#define SIGNAL_RELEASE(SIG) \
+	{	sigset_t	sigmask;	\
+		sigemptyset(&sigmask);		\
+		sigaddset(&sigmask,SIG);	\
+		sigprocmask(SIG_UNBLOCK,&sigmask,NULL);	\
+	}
+#define SIGNAL_IGNORE(SIG) \
+	SIGNAL_HANDLE(SIG,SIG_IGN)
+#endif /* no SA_INTERRUPT */
 	
 #else	/* No SIGACTION (POSIX), nor SIGVEC stuff */
 
@@ -89,3 +124,7 @@
 
 #endif
 #endif
+
+#ifndef SIGCHLD
+#define SIGCHLD SIGCLD
+#endif
