@@ -12,10 +12,13 @@ univptr_t
 _mal_sbrk(nbytes)
 size_t nbytes;
 {
-	static char *lastsbrk;
+	static char *lastsbrk = NULL;
 	char *p;
 
 	p = (char *) sbrk((int) nbytes);
+	if (p == (char*)-1L) {
+	  return p; /* Ran out of memory! */
+	}
 	/*
 	 * Some old SVR3 apparently have a kernel bug: after several sbrk
 	 * calls, sbrk suddenly starts returning values lower than the ones it
