@@ -507,14 +507,14 @@ dohup()
 
 struct de {
 	int		f_name;
-	struct stat	*f_stat;
+	time_t		mtime;
 };
 
 static int
 decmp(a, b)
 	register struct de *a, *b;
 {
-	return b->f_stat->st_mtime - a->f_stat->st_mtime;
+	return b->mtime - a->mtime;
 }
 
 static int desize, nbsize;
@@ -701,13 +701,7 @@ rd_stability(dirp,dirs)
 		 * management code and might still do bad things with
 		 * the malloc algorithm.
 		 */
-		dearray[deindex].f_stat =
-			(struct stat *)emalloc(sizeof (struct stat));
-		if (stat(pathbuf, dearray[deindex].f_stat) < 0) {
-			perror(pathbuf);
-			free((char *)dearray[deindex].f_stat);
-			continue;
-		}
+		dearray[deindex].mtime = statbuf.st_mtime;
 		dearray[deindex].f_name = nbindex;
 		memcpy(nbarray + nbindex, dp->d_name, namelen+1);
 		nbindex += namelen + 1;
@@ -715,16 +709,11 @@ rd_stability(dirp,dirs)
 		++deindex;
 	}
 	if (mustexit) {
-		for (i = 0; i < deindex; ++i)
-			free((char *)dearray[i].f_stat);
 		return deindex;
 	}
 
 	qsort((char *)dearray, deindex, sizeof dearray[0], decmp);
 
-	for (i = 0; i < deindex; ++i)
-		free((char *)dearray[i].f_stat);
-
 	did_cnt = 0;
 	while (deindex-- > 0) {
 		if (mustexit)
