@@ -209,6 +209,32 @@ print_ndbm(sip, outfp)
 	fflush(outfp);
 }
 
+/*
+ * Count the database.
+ */
+
+void
+count_ndbm(sip, outfp)
+	struct search_info *sip;
+	FILE *outfp;
+{
+	DBM *db;
+	datum key, val;
+	int cnt = 0;
+
+	if ((db = open_ndbm(sip, O_RDONLY, "count_ndbm")) != NULL)
+	  for (key = dbm_firstkey(db); key.dptr != NULL; key = dbm_nextkey(db)) {
+	    val = dbm_fetch(db, key);
+	    if (val.dptr == NULL)
+	      continue;
+	    if (dbm_error(db))
+	      break;
+	    ++cnt;
+	}
+	fprintf(outfp,"%d\n",cnt);
+	fflush(outfp);
+}
+
 /*
  * Print the uid of the owner of the database.  Note that for ndbm-style
  * databases there are several files involved so picking one of them for
