@@ -35,14 +35,12 @@ doagenda()
 	     should be traversed only once, because  thread_start kicks
 	     right away (or reschedules) */
 
-	  while (ncuritem != NULL /* && ncuritem->wakeup <= now */) {
+	  while (ncuritem != NULL /* && ncuritem->wakeup <= now */ ) {
 	    /* Object pointed by  ncuritem  may disappear due to
 	       expiration at thread_start() .. */
 	    nncuritem = ncuritem->nexttr;
-	    if (ncuritem->proc == NULL && ncuritem->wakeup <= now) {
+	    if (ncuritem->proc == NULL && ncuritem->wakeup <= now)
 	      didsomething += thread_start(ncuritem);
-	      queryipccheck();
-	    }
 
 	    ncuritem = nncuritem;
 	  }
