@@ -88,6 +88,8 @@ extern int  vtxredo __((struct spblk *spl));
 extern char *timestring __((void));
 extern time_t now;
 extern FILE *statuslog;
+extern void resync_file __((char *filename));
+extern int thread_count_recipients __((void));
 
 /* threads.c */
 extern struct thread *thread_head, *thread_tail;
@@ -130,4 +132,4 @@ extern char *saytime __((long, char *, int));
 extern void expire __((struct vertex *, int));
 
 /* Transport library */
-extern int lockaddr __((int, long, char, char));
+extern int lockaddr __((int, char *, long, char, char));
