@@ -113,6 +113,7 @@ qprint(fd)
 	}
 }
 
+static int qpctlfile __((struct spblk *spl));
 static int qpctlfile(spl)
 	struct spblk *spl;
 {
@@ -188,6 +189,7 @@ static int qpctlfile(spl)
 	  return 0;
 	}
 
+static int qpchannel __((struct spblk *spl));
 static int qpchannel(spl)
 	struct spblk *spl;
 {
@@ -197,12 +199,16 @@ static int qpchannel(spl)
 	if (wc->link != NULL) {
 	  fprintf(qpfp, "%s:\t", wc->name);
 	  for (vp = wc->link; vp != NULL; vp = vp->next[L_CHANNEL])
-	    fprintf(qpfp, ">%d", vp->qid);
+	    if (vp->qid)
+	      fprintf(qpfp, ">%d", vp->qid);
+	    else
+	      fprintf(qpfp, ">999%04d", vp->cfp->id);
 	  fprintf(qpfp, "\n");
 	}
 	return 0;
 }
 
+static int qphost __((struct spblk *spl));
 static int qphost(spl)
 	struct spblk *spl;
 {
@@ -212,7 +218,10 @@ static int qphost(spl)
 	if (wc->link != NULL) {
 	  fprintf(qpfp, "%s:\t", wc->name);
 	  for (vp = wc->link; vp != NULL; vp = vp->next[L_HOST])
-	    fprintf(qpfp, ">%d", vp->qid);
+	    if (vp->qid)
+	      fprintf(qpfp, ">%d", vp->qid);
+	    else
+	      fprintf(qpfp, ">999%04d", vp->cfp->id);
 	  fprintf(qpfp, "\n");
 	}
 	return 0;
