@@ -116,8 +116,6 @@ extern void die __((int rc, char *reason));
 extern int  killprevious __((int signal, char *pidfile));
 extern void prversion __((char *name));
 
-extern int thread_count_recipients();
-
 struct dirstatname {
 	struct stat st;
 	long ino;
@@ -136,24 +134,35 @@ int global_maxkids = 1000;
 time_t now;
 FILE *statuslog = NULL;
 
-void cfp_free(cfp)
+static void cfp_free __((struct ctlfile *cfp, struct spblk *spl));
+static void cfp_free(cfp, spl)
 struct ctlfile *cfp;
+struct spblk   *spl;
 {
 	struct vertex *vp, *nvp;
-	struct spblk *spl;
+	int resyncing = (spl != NULL);
 
 	/* Delete from the  spt_mesh[]  */
 
-	spl = sp_lookup(cfp->id, spt_mesh[L_CTLFILE]);
+	if (spl == NULL)
+	  spl = sp_lookup((u_long)(cfp->id), spt_mesh[L_CTLFILE]);
 	if (spl != NULL)
 	  sp_delete(spl, spt_mesh[L_CTLFILE]);
 
 	/* And from memory */
 
+	if (cfp->vfp) {
+	  fprintf(cfp->vfp,
+		  "ordered deletion of task file from scheduler memory (%s)\n",
+		  cfp->mid);
+	  fclose(cfp->vfp);
+	}
+
 	if (cfp->head != NULL) {
 	  for (vp = cfp->head; vp != NULL; vp = nvp) {
 	    nvp = vp->next[L_CTLFILE];
-	    unvertex(vp,1,1); /* Don't unlink! Just free()! */
+	    vp->ngroup = 0;
+	    unvertex(vp,1,1); /* Don't unlink()! Just free()! */
 	  }
 	} else {
 	  free_cfp_memory(cfp);
@@ -164,7 +173,7 @@ static int ctl_free __((struct spblk *spl));
 static int ctl_free(spl)
 struct spblk *spl;
 {
-	cfp_free((struct ctlfile *)spl->data);
+	cfp_free((struct ctlfile *)spl->data, NULL);
 	return 0;
 }
 
@@ -570,7 +579,7 @@ static int in_dirscanqueue(dq,ino)
 	/* Return 1, if can find the "ino" in the queue */
 
 	if (dq->wrkcount == 0 || dirscan_mesh == NULL) return 0;
-	if (sp_lookup(ino,dirscan_mesh) != NULL) return 1;
+	if (sp_lookup((u_long)ino,dirscan_mesh) != NULL) return 1;
 	return 0;
 }
 
@@ -630,7 +639,7 @@ if (verbose) { printf("dirqueuescan(dir='%s') ",dir); fflush(stdout); }
 	    /* We may have this file in processing state...  */
 	    {
 	      struct spblk *spl;
-	      spl = sp_lookup(ino, spt_mesh[L_CTLFILE]);
+	      spl = sp_lookup((u_long)ino, spt_mesh[L_CTLFILE]);
 	      if (spl != NULL) {
 		/* Already in processing, don't touch.. */
 		/*printf("File: %s active (not previously locked)\n",file);*/
@@ -718,13 +727,13 @@ static int syncweb(dq, cehead)
 	/* Now we have pointers */
 
 	/* Deletion from the  dirscan_mesh  should ALWAYS succeed.. */
-	sp_delete(sp_lookup(ino,dirscan_mesh),dirscan_mesh);
+	sp_delete(sp_lookup((u_long)ino,dirscan_mesh),dirscan_mesh);
 
 	/* Sometimes we may get files already in processing
 	   into our pre-schedule queue */
 
 	/* Already in processing ? */
-	spl = sp_lookup(ino, spt_mesh[L_CTLFILE]);
+	spl = sp_lookup((u_long)ino, spt_mesh[L_CTLFILE]);
 	if (spl == NULL) {
 	  /* Not yet in processing! */
 	  int fd;
@@ -750,6 +759,35 @@ static int syncweb(dq, cehead)
 	return wrkcnt;
 }
 
+void resync_file(file)
+	char *file;
+{
+	struct stat stbuf;
+	struct spblk *spl;
+	long ino, fd;
+
+	lstat(file,&stbuf);
+	ino = atoi(file);
+
+	/* Sometimes we may get reports of files
+	   already deleted from processing.. */
+	printf("Resyncing file \"%s\" (ino=%d)",file,ino);
+
+	/* Already in processing ? */
+	spl = sp_lookup((u_long)ino, spt_mesh[L_CTLFILE]);
+	if (spl == NULL) {
+	  printf(" .. not in processing database\n");
+	  /* Not (anymore) in processing! */
+	  return;
+	}
+	printf(" .. in processing db\n");
+
+	/* Delete it... */
+	cfp_free((struct ctlfile *)spl->data, spl);
+	/* It will return into processing at next queue scan.. */
+}
+
+
 /*
  * The schedule() function is in charge of reading the control file from
  * the scheduler directory, and creating all the appropriate links to the
@@ -950,7 +988,7 @@ lockverify(cfp,cp)		/* Return 1 when lock process does not exist */
 	if (kill(lockpid,sig) != 0) return 1; /* PID does not exist, or
 					       other error.. */
 	fprintf(stderr,"lockverify: Lock with PID=%d is active on %s:%s\n",
-		lockpid, cfp->logident, cp+_CFTAG_RCPTPIDSIZE);
+		lockpid, cfp->mid, cp+_CFTAG_RCPTPIDSIZE);
 	return 0;	/* Locking PID does exist.. */
 }
 
@@ -1014,14 +1052,14 @@ static struct ctlfile *vtxprep(cfp)
 			"New scheduler: Skipped a job-file because it is held locked by PID=%6.6s\n",cp+1);
 		fclose(cfp->vfp);
 	      }
-	      cfp_free(cfp);
+	      cfp_free(cfp, NULL);
 	      ++vtxprep_skip;
 	      ++vtxprep_skip_lock;
 	      free(offarr);
 	      return NULL;
 	    }
 	    *cp = _CFTAG_NORMAL; /* unlock it */
-	    lockaddr(cfp->fd, (long) (cp - cfp->contents),
+	    lockaddr(cfp->fd, NULL, (long) (cp - cfp->contents),
 		     _CFTAG_LOCK, _CFTAG_NORMAL);
 	  }
 	  /* Calculate summary info */
@@ -1188,7 +1226,7 @@ static struct ctlfile *vtxprep(cfp)
 		    "aborted due to missing information\n");
 	    fclose(cfp->vfp);
 	  }
-	  cfp_free(cfp);
+	  cfp_free(cfp, NULL);
 	  free(offarr);
 	  return NULL;
 	}
@@ -1346,6 +1384,13 @@ static struct ctlfile *vtxprep(cfp)
 	*/
 	cfp->head = head;
 	free(offarr);
+	if (verbose) {
+	  int completed = cfp->rcpnts_total - cfp->rcpnts_work -
+			  cfp->rcpnts_failed;
+	  printf("vtxprep: msg %s rcptns total %d work %d failed %d done %d\n",
+		 cfp->mid, cfp->rcpnts_total, cfp->rcpnts_work,
+		 cfp->rcpnts_failed, completed );
+	}
 	return cfp;
 }
 
