@@ -8,7 +8,7 @@
  *	Matti Aarnio <mea@nic.funet.fi>  (copyright) 1992-1995
  */
 
-#define USE_SIGREAPER /* DON'T Use SIGCLD-driven reaper.. */
+#define USE_SIGREAPER /* DO Use SIGCLD-driven reaper.. */
 
 #include "hostenv.h"
 #include "malloc.h"
