@@ -529,8 +529,8 @@ unweb(flag, wp)
 /*
  * unthread(vtx) -- detach this vertex from its thread
  */
-void
-unthread(vtx)
+static void unthread __((struct vertex *vtx));
+static void unthread(vtx)
 struct vertex *vtx;
 {
 	struct thread *thr = vtx->thread;
@@ -598,8 +598,8 @@ struct vertex **a, **b;
 }
 
 
-void
-thread_vertex_shuffle(thr)
+static void thread_vertex_shuffle __((struct thread *thr));
+static void thread_vertex_shuffle(thr)
 struct thread *thr;
 {
 	register struct vertex *vp;
