@@ -63,10 +63,18 @@ void
 feed_child(proc)
 struct procinfo *proc;
 {
-	struct vertex *vtx = proc->vertex;
+	struct vertex *vtx;
 	int len, rc;
 
-	if (proc->thread == NULL) return; /* Might be called without next process.. */
+	/* SIGNAL_HOLD(SIGCHLD); */
+	if (proc->thread == NULL) {
+	  /* SIGNAL_RELEASE(SIGCHLD); */
+	  return; /* Might be called without next process.. */
+	}
+	if (proc->pid <= 0 || proc->tofd < 0) {
+	  /* SIGNAL_RELEASE(SIGCHLD); */
+	  return; /* No process../No write channel.. */
+	}
 
 	if (proc->cmdbuf[0] != 0) {
 	  /* We have some leftovers from previous feed..
@@ -79,11 +87,11 @@ struct procinfo *proc;
 	      /* Some real failure :-( */
 	      pipes_shutdown_child(proc->tofd);
 	      proc->tofd = -1;
-	      vtx->proc = NULL;
-	      vtx       = NULL;
+	      proc->vertex->proc = NULL;
 	      proc->thread->proc = NULL;
 	      proc->thread       = NULL;
 	      proc->cmdbuf[0] = 0;
+	      /* SIGNAL_RELEASE(SIGCHLD); */
 	      return;
 	    }
 	    if (rc < 0) rc = 0;
@@ -94,9 +102,11 @@ struct procinfo *proc;
 	    proc->cmdbuf[0] = 0; /* Clean the pending info.. */
 	  }
 	  proc->feedtime = now;
+	  /* SIGNAL_RELEASE(SIGCHLD); */
 	  return; /* We return latter.. */
 	}
 
+	vtx = proc->vertex;
 	vtx->proc = proc;    /* Flag that it is in processing */
 	vtx->ce_pending = 0; /* and clear the pending.. */
 
@@ -115,6 +125,7 @@ struct procinfo *proc;
 	  proc->cmdbuf[0] = 0;
 	  if (verbose)
 	    printf(" ... no refeeding\n");
+	  /* SIGNAL_RELEASE(SIGCHLD); */
 	  return;
 	}
 
@@ -145,7 +156,9 @@ struct procinfo *proc;
 	proc->hungry = 0;
 	proc->fed = 1;
 	proc->feedtime = now;
-	vtx->attempts += 1;
+	if (vtx)
+	  vtx->attempts += 1; /* We may get it closed above.. */
+	/* SIGNAL_RELEASE(SIGCHLD); */
 }
 
 /*
@@ -322,36 +335,9 @@ static void stashprocess(pid, fromfd, tofd, chwp, howp, vhead)
 	time(&proc->hungertime); /* Actually it is not yet 'hungry' as
 				    per reporting so, but we store the
 				    time-stamp anyway */
-#ifdef O_NONBLOCK /* POSIXy thing */
-	/* set up non-blocking I/O */
-	i = fcntl(fromfd, F_GETFL, 0);
-	i |= O_NONBLOCK;
-	i = fcntl(fromfd, F_SETFL, i);
-	if (fromfd != tofd) {
-	  i = fcntl(tofd, F_GETFL, 0);
-	  i |= O_NONBLOCK;
-	  i = fcntl(tofd, F_SETFL, i);
-	}
-#else
-#ifdef	USE_BSDNODELAY
-	i = fcntl(fromfd, F_GETFL, 0);
-#ifdef	FNONBLOCK
-	i |= FNONBLOCK;
-#else
-	i |= FNDELAY;
-#endif
-	i = fcntl(fromfd, F_SETFL, i);
-	if (fromfd != tofd) {
-	  i = fcntl(tofd, F_GETFL, 0);
-#ifdef	FNONBLOCK
-	  i |= FNONBLOCK;
-#else
-	  i |= FNDELAY;
-#endif
-	  i = fcntl(tofd, F_SETFL, i);
-	}
-#endif	/* USE_BSDNODELAY */
-#endif	/* O_NONBLOCK  */
+	fd_nonblockingmode(fromfd);
+	if (fromfd != tofd)
+	  fd_nonblockingmode(tofd);
 }
 
 static void reclaim(fromfd, tofd)
@@ -457,7 +443,6 @@ static void waitandclose(fd)
 	   - fd returns -1, and errno != {EAGAIN|EWOULDBLOCK|EINTR}
 	 */
 	reclaim(fd, cpids[fd].tofd);
-
 }
 
 #ifdef	USE_SELECT
