@@ -99,6 +99,9 @@ update(fd, diagnostic)
 	}
 	if (*diagnostic == '#' /* DEBUG comment.. */) {
 	  if (strncmp(diagnostic,"#hungry",7)==0) {
+	    /* This is an "actor model" behaviour,
+	       where actor tells, when it needs a new
+	       job fed to it. */
 	    if (proc->tofd   >= 0) {
 	      proc->hungry = 1;
 	      time(&proc->hungertime);
@@ -142,6 +145,19 @@ update(fd, diagnostic)
 		printf("'#hungry' from child without forward-channel\n");
 	    return;
 	  } /* end of '#hungry' processing */
+	  if (strncmp(diagnostic,"#resync",7)==0) {
+	    /* The transporter has noticed that the scheduler
+	       gave it a job spec, which does not have anything
+	       left for processing, it is time for the scheduler
+	       to recheck the job file. */
+	    char *p, *s = diagnostic + 7;
+	    while (*s == ' ' || *s == '\t') ++s;
+	    p = strchr(s,'\n');
+	    if (p) *p = 0; /* newline AFTER filename */
+	    if (*s != 0)
+	      resync_file(s);
+	    return;
+	  }
 	  fprintf(stderr, "%s DBGdiag: %s\n", timestring(), diagnostic);
 	  return;
 	} /* end of debug-diagnostics processing */
