@@ -4,7 +4,7 @@
  */
 /*
  *	Several extensive changes by Matti Aarnio <mea@nic.funet.fi>
- *	Copyright 1991-1995.
+ *	Copyright 1991-1996.
  */
 
 /*
@@ -170,6 +170,7 @@ int	skeptical  = 1;
 int	checkhelo  = 0;
 int	verbose    = 0;
 int	daemon_flg = 1;
+int	rcpt_count = 0;
 int	pid, routerpid;
 FILE	*logfp = NULL;
 int	D_alloc = 0;
@@ -200,15 +201,13 @@ long availspace = -1;	/* available diskspace/2 in bytes	*/
 long maxsize    = 0;
 long sizeoptval = 0;	/* "MAIL FROM:<xxx> SIZE=nnn" -value	*/
 long sizeoptsum = 0;
-char buf[SMTPLINESIZE];	/* limits size of SMTP commands...
-			   On the other hand, limit is asked
-			   to be only 1000 chars, not 8k.. */
-char helobuf[SMTPLINESIZE];
 int maxloadavg = 999;	/* Maximum load-average that is tolerated
 			   with smtp-server actively receiving..
 			   Default value of 999 is high enough
 			   so that it will never block -- use
 			   "-L 10" to define lower limit (10) */
+char helobuf[SMTPLINESIZE];
+
 
 
 
@@ -292,6 +291,7 @@ int insecure;
 					progname, logfile, strerror(errno));
 		} else {
 			logfp = fdopen(fd, "a");
+/* XX: */		setbuf(logfp,NULL);
 		}
 	} else
 		logfp = NULL;
@@ -310,6 +310,8 @@ main(argc, argv)
 #endif	/* USE_INET */
 	u_short port = 0;
 
+	SIGNAL_HANDLE(SIGPIPE, SIG_IGN);
+
 	daemon_flg = 1;
 	inetd = errflg = version = 0;
 	logfile = NULL;
@@ -760,10 +762,61 @@ reporterr(tell, msg)
 	}
 }
 
-void
-smtp_helo(cp)
-char *cp;
+
+/* Support routine: Our own buffering for stdinput */
+
+int  s_bufread = 0;
+int  s_readout = 0;
+char s_buffer[SMTPLINESIZE];
+int  s_inputfd = 0; /* STDIN */
+int  s_status  = 0;
+
+int s_feof()
 {
+	return s_status;
+}
+
+int s_getc()
+{
+	int rc = 0;
+
+	if (s_status) return s_status;
+	if (s_readout >= s_bufread) {
+	  int rc = read(s_inputfd, s_buffer, sizeof(s_buffer));
+	  if (rc <= 0) {
+	    s_status = EOF;
+	    return EOF;
+	  }
+	  s_bufread = rc;
+	  s_readout = 0;
+	}
+	if (rc) return EOF;
+	return (unsigned char) s_buffer[s_readout++];
+}
+
+int s_hasinput()
+{
+	if (s_readout >= s_bufread) return 0;
+	return (s_bufread - s_readout);
+}
+
+void s_setup(fd)
+	int fd;
+{
+	s_status = 0;
+	s_inputfd = fd;
+	s_bufread = -1;
+	s_readout = 0;
+}
+
+
+/* SMTP-server verb subroutines */
+
+void
+smtp_helo(buf,cp)
+char *buf, *cp;
+{
+
 	if (state != Hello && state != MailOrHello) {
 	  switch (state) {
 	    case Mail:
@@ -797,11 +850,24 @@ char *cp;
 	 */
 	if (checkhelo && skeptical && partridge(helobuf, cfinfo)) {
 	  rfc821_error_ptr += (cp - helobuf);
-	  type821err(501, buf,
-		     "Invalid `%s' parameter!  Err: %s",
-		     buf, rfc821_error);
+	  type821err(-501, buf,
+		     "Invalid `%s' parameter!",
+		     buf);
+	  type(501,"Err: %s", rfc821_error);
+	  strcpy(helobuf,"Bad.Helo.Input");
 	  return;
 	}
+	/* At least check the input, though say "Ok, Master" while
+	   complaining.. */
+	if (!checkhelo && partridge(helobuf, cfinfo)) {
+	  rfc821_error_ptr += (cp - helobuf);
+	  type821err(-250, buf, "Invalid `%s' parameter!", buf);
+	  type(-250,"Err: %s", rfc821_error);
+	  if (carp->cmd == Hello)
+	    strcpy(helobuf,"Bad.HELO.Input");
+	  else
+	    strcpy(helobuf,"Bad.EHLO.Input");
+	}
 	cfinfo = findcf(helobuf);
 	if (cfinfo != NULL && *(cfinfo->flags) == '!') {
 	  if (cfinfo->flags[1] != '\0')
@@ -815,17 +881,17 @@ char *cp;
 	    && rhostname[0] != '\0' && rhostname[0] != '['
 	    && cistrcmp(helobuf, rhostname) != 0) {
 	  if (checkhelo) {
-	    type(-250, " %s That hostname is inconsistent with",
+	    type(-250, "%s That hostname is inconsistent with",
 	         myhostname);
-	    type(-250, " %s your address to name mapping.",
+	    type(-250, "%s your address to name mapping.",
 	         myhostname);
 	  }
 	  type(carp->cmd==Hello2 ? -250: 250,
-	       " %s expected \"%s %s\"", myhostname,
+	       "%s expected \"%s %s\"", myhostname,
 	     carp->verb, rhostname);
 	} else
 	  type(carp->cmd==Hello2 ? -250: 250,
-	       " %s Hello %s", myhostname, helobuf);
+	       "%s Hello %s", myhostname, helobuf);
 	if (carp->cmd == Hello2) {
 
 	  /* ESMTP -- RFC 1651,1652,1653,1428 thingies */
@@ -836,9 +902,11 @@ char *cp;
 						  The FIXED maximum */
 	  type(-250,sizebuf);
 	  type(-250,"8BITMIME");
+	  type(-250,"PIPELINING");
 	  type(-250,"EXPN");
 	  type(-250,"VRFY");
 	  type(-250,"XDSN"); /* XX: Change to "DSN" when all parts of it function! */
+	  type(-250,"X-RCPTLIMIT 10000"); /* VERY HIGH figure, normal is 100 */
 	  type( 250,"HELP");
 	  with_protocol = WITH_ESMTP;
 	}
@@ -846,8 +914,8 @@ char *cp;
 }
 
 void
-smtp_mail(cp,insecure)
-char *cp;
+smtp_mail(buf,cp,insecure)
+char *buf, *cp;
 int insecure;
 {
 	char *s;
@@ -1053,11 +1121,12 @@ int insecure;
 	if (s)
 	  free(s);
 	state = Recipient;
+	rcpt_count = 0;
 }
 
 void
-smtp_rcpt(cp)
-char *cp;
+smtp_rcpt(buf,cp)
+char *buf, *cp;
 {
 	char *s;
 	char *drpt_notify, *drpt_ret, *drpt_orcpt;
@@ -1225,29 +1294,28 @@ char *cp;
 	    type(250, "Ok; can accomodate %d byte message",sizeoptval);
 	  else
 	    type(250, (char *)NULL);
+	  ++rcpt_count;
 	}
 	if (s)
 	  free(s);
 	state = RecipientOrData;
 }
 
+/* The SMTP-server itself */
 
 void
 smtpserver(insecure)
 	int insecure;
 {
-	char *cp, *s;
+	char *cp;
 	time_t now;
 	long tell, filsiz;
 	int inum, cfi;
 	struct stat stbuf;
 	struct sockaddr_in localsock;
 	int localsocksize;
-	int rc;
-	int c, co;
 	char msg[2048];
 
-	char *eobuf;
 	int VerboseCommand = 0;
 
 	openlogfp();
@@ -1261,8 +1329,10 @@ smtpserver(insecure)
 	if (*(cp + strlen(cp) - 1) == '\n')
 	  *(cp + strlen(cp) - 1) = '\0';
 
+	s_setup(FILENO(stdin));
+
 	localsocksize = sizeof(localsock);
-	if (getsockname(FILENO(stdin),
+	if (getsockname(s_inputfd,
 			(struct sockaddr *)&localsock,&localsocksize) == 0 &&
 	    localsocksize == sizeof(localsock)) {
 	  /* Success with it :) */
@@ -1274,6 +1344,8 @@ smtpserver(insecure)
 
 	type(220, "%s Server ESMTP%s %s ready at %s",
 	     myhostname, auth_flag ? "+AUTH":"", VersionNumb, cp);
+	fflush(stdout);
+
 #ifdef	LOG_PID
 #ifdef	LOG_MAIL
 	openlog("smtpserver", LOG_PID|LOG_CONS, LOG_MAIL);
@@ -1302,33 +1374,37 @@ smtpserver(insecure)
 
 	while (1) {
 
+	  char buf[SMTPLINESIZE];	/* limits size of SMTP commands...
+					   On the other hand, limit is asked
+					   to be only 1000 chars, not 8k.. */
+	  int rc = -1;
+	  int c, co = -1;
+	  int i;
+	  char *eobuf;
+
 	  /* Alarm processing on the SMTP protocol channel */
 	  alarm(SMTP_COMMAND_ALARM_IVAL);
 
-	  *buf = 0;
-	  s = buf;
-	  eobuf = buf + sizeof(buf) -1;
-	  rc = -1;
-	  co = -1;
 	  /* Our own  fgets() -- gets also NULs, flags illegals.. */
-	  while ((c = getc(stdin)) != EOF && s < eobuf) {
+	  i = 0;
+	  while ((c = s_getc()) != EOF && i < (sizeof(buf)-1)) {
 	    if (c == '\n') {
 	      /* *s++ = c; */ /* Don't save it! No need */
 	      break;
 	    } else if (co == '\r' && rc < 0)
-	      rc = (s - buf);	/* Spurious CR on the input.. */
+	      rc = i;	/* Spurious CR on the input.. */
 
-	    if (c == '\0'       && rc < 0) rc = (s - buf);
-	    if ((c & 0x80) != 0 && rc < 0) rc = (s - buf);
+	    if (c == '\0'       && rc < 0) rc = i;
+	    if ((c & 0x80) != 0 && rc < 0) rc = i;
 	    if (c != '\r' && c != '\t' &&
 		(c < 32 || c == 127) && rc < 0)
-	      rc = (s - buf);
-	    *s++ = c;
+	      rc = i;
+	    buf[i++] = c;
 	  }
-	  *s = '\0';
-	  eobuf = s;		/* Buf end ptr.. */
+	  buf[i] = '\0';
+	  eobuf = &buf[i];	/* Buf end ptr.. */
 	  alarm(0);		/* Cancel the alarm */
-	  if (c == EOF && s == buf) {
+	  if (c == EOF && i == 0) {
 	    /* XX: ???  Uh, Hung up on us ? */
 	    if (mfp != NULL)
 	      mail_abort(mfp);
@@ -1361,6 +1437,7 @@ smtpserver(insecure)
 		       ((buf[rc]=='\0') ? "NUL on SMTP input" :
 			((buf[rc]&0x80) ? "8-bit char on SMTP input":
 			 "Control chars on SMTP input")));
+	    fflush(stdout);
 	    continue;
 	  }
 
@@ -1372,6 +1449,7 @@ smtpserver(insecure)
 	    continue;
 	  if (cp >  buf + 6) {	/* "DEBUG" is longest of them.. */
 	    type(550, "Syntax error");
+	    fflush(stdout);
 	    continue;
 	  }
 	  c = *cp;
@@ -1389,16 +1467,19 @@ smtpserver(insecure)
 		   "unknown SMTP command '%s' from %s/%d",
 		   buf, rhostname, rport);
 #endif /* LOG_WARNING */
+	    fflush(stdout);
 	    continue;
 	  }
 	  switch (carp->cmd) {
 	    case Null:
 		type(550, "panic!");
+		fflush(stdout);
 		break;
 	    case Hello:
 	    case Hello2:
 		/* This code is LONG.. */
-		smtp_helo(cp);
+		smtp_helo(buf,cp);
+		fflush(stdout);
 		break;
 	    case Mail:
 	    case Mail2:
@@ -1407,11 +1488,19 @@ smtpserver(insecure)
 	    case SendOrMail:
 	    case SendAndMail:
 		/* This code is LONG.. */
-		smtp_mail(cp,insecure);
+		smtp_mail(buf,cp,insecure);
+		if (!s_hasinput())
+		  fflush(stdout);
+	        else
+		  fprintf(logfp,"%d#\t-- pipeline input exists %d bytes\n", pid, s_hasinput());
 		break;
 	    case Recipient:
 		/* This code is LONG.. */
-		smtp_rcpt(cp);
+		smtp_rcpt(buf,cp);
+		if (!s_hasinput())
+		  fflush(stdout);
+		else
+		  fprintf(logfp,"%d#\t-- pipeline input exists %d bytes\n", pid, s_hasinput());
 		break;
 	    case Data:
 		if (state != RecipientOrData) {
@@ -1431,9 +1520,19 @@ smtpserver(insecure)
 			break;
 		  }
 		  type(503, cp);
+		  fflush(stdout);
 		  break;
 		}
+		if (rcpt_count == 0) {
+		  /* No valid recipients! */
+		  type(550, "No valid recipient at RCPT addresses, or no RCPT addresses at all");
+		  fflush(stdout);
+		  break;
+		}
+		rcpt_count = 0; /* now we can zero it.. */
 		type(354, (char *)NULL);
+		fflush(stdout);
+		fputs("env-end\n",mfp);
 		/* We set alarm()s inside the mvdata() */
 		*msg = 0;
 		filsiz = mvdata(mfp, msg);
@@ -1448,14 +1547,17 @@ smtpserver(insecure)
 		  mail_abort(mfp);
 		  mfp = NULL;
 		  type(452,"%s",msg);
-		} else if (feof(stdin) || ferror(stdin)) {
+		  fflush(stdout);
+		} else if (s_feof()) {
 		  /* [mea@utu.fi] says this can happen */
 		  mail_abort(mfp);
 		  mfp = NULL;
 		  reporterr(tell, "premature EOF on DATA input");
+		  fflush(stdout);
 		  return;
 		} else if (ferror(mfp)) {
 		  type(452, (char *)NULL);
+		  fflush(stdout);
 		  clearerr(mfp);
 		  mail_abort(mfp);
 		  mfp = NULL;
@@ -1464,14 +1566,17 @@ smtpserver(insecure)
 		  mail_abort(mfp);
 		  mfp = NULL;
 		  type(552, "Message size exceeded fixed maximum size for acceptable email");
+		  fflush(stdout);
 
 		} else if (mail_close(mfp) == EOF) {
 		  type(452, (char *)NULL);
+		  fflush(stdout);
 		  mfp = NULL;
 		  reporterr(tell, "message file close failed");
 		} else {
 		  mfp = NULL;
 		  type(250, "Roger");
+		  fflush(stdout);
 #ifdef	LOG_INFO
 		  syslog(LOG_INFO,
 			 "accepted id %d (%ldc) from %s/%d",
@@ -1485,6 +1590,7 @@ smtpserver(insecure)
 		  }
 		}
 		state = MailOrHello;
+		fflush(stdout);
 		break;
 	    case Reset:
 		if (mfp != NULL) {
@@ -1494,17 +1600,21 @@ smtpserver(insecure)
 		if (state != Hello)
 		  state = MailOrHello;
 		type(250, (char *)NULL);
+		fflush(stdout);
 		break;
 	    case Help:
 		help(cfinfo, cp);
+		fflush(stdout);
 		break;
 	    case Verify:
 	    case Verify2:
 		if (state == Hello) {
 		  type (503, "Waiting for HELO/EHLO command");
+		  fflush(stdout);
 		  break;
 		}
 		if ((cfi = STYLE(cfinfo, 'v'))) {
+		  char *s;
 		  while (*cp == ' ' || *cp == '\t') ++cp;
 		  if (*cp == '<')
 		    s = rfc821_path(cp,cfi); /* with < > */
@@ -1525,13 +1635,16 @@ smtpserver(insecure)
 		  }
 		} else
 		  type(252, (char *)NULL);
+		fflush(stdout);
 		break;
 	    case Expand:
 		if (state == Hello) {
 		  type (503, "Waiting for HELO/EHLO command");
+		  fflush(stdout);
 		  break;
 		}
 		if ((cfi = STYLE(cfinfo, 'e'))) {
+		  char *s;
 		  while (*cp == ' ' || *cp == '\t') ++cp;
 		  if (*cp == '<')
 		    s = rfc821_path(cp,cfi); /* with < > */
@@ -1552,30 +1665,37 @@ smtpserver(insecure)
 		  }
 		} else
 		  type(502, (char *)NULL);
+		fflush(stdout);
 		break;
 	    case Turn:
 		type(502, (char *)NULL);
+		fflush(stdout);
 		break;
 	    case NoOp:
 		type(250, (char *)NULL);
+		fflush(stdout);
 		break;
 	    case Verbose:
 		type(-250, VerbID, Version);
 		type(-250, Copyright);
 		type(250, Copyright2);
+		fflush(stdout);
 		VerboseCommand = 1;
 		break;
 	    case DebugMode:
 		debug_report(VerboseCommand,rhostname,buf);
+		fflush(stdout);
 		break;
 	    case Tick:
 		type(250, "%s", buf);
+		fflush(stdout);
 		with_protocol = WITH_BSMTP;
 		break;
 	    case Quit:
 		if (mfp != NULL)
 		  mail_abort(mfp);
 		type(221, (char *)NULL, "Out");
+		fflush(stdout);
 		return;
 	    default:
 		break;
@@ -1583,7 +1703,7 @@ smtpserver(insecure)
 	}
 	if (mfp != NULL)
 	  mail_abort(mfp);
-	reporterr(ftell(stdin), "session terminated");
+	reporterr(lseek(0,0,SEEK_CUR), "session terminated");
 }
 
 #define	TYPE_(m)	type(-214, "%s", m);
@@ -2127,7 +2247,6 @@ type(code, fmt, s1, s2, s3, s4, s5)
 	else
 		printf(text, s1, s2, s3, s4, s5);
 	printf("\r\n");
-	fflush(stdout);
 	if (logfp != NULL) {
 		if (fmt != NULL)
 			fprintf(logfp, fmt, s1, s2, s3, s4, s5);
@@ -2147,7 +2266,7 @@ type821err(code,inbuf,msg,s1,s2,s3)
 {
 	char *s = inbuf+4;
 
-	printf("%03d-",code);
+	printf("%03d-", code < 0 ? -code : code);
 	if (logfp != NULL)
 	  fprintf(logfp, "%dw\t%03d-", pid, code);
 	if (strlen(inbuf)>3)
@@ -2243,7 +2362,6 @@ mvdata(out,msg)
 	register FILE *out;
 	char *msg;
 {
-	register FILE *input;
 	register int c, state, *idxnum, *sts, endstate, cnt;
 	int timercnt = 1024;
 
@@ -2252,7 +2370,6 @@ mvdata(out,msg)
 
 	state = 10;
 	endstate = X_;
-	input = stdin;
 	sts = states;
 	cnt = 0;
 #else
@@ -2265,11 +2382,12 @@ mvdata(out,msg)
 	int linecnt = 0;
 	int was8bit = 0;
 
+	fflush(stdout);
+
 	idxnum = indexnum+1;
 
 	state = 10;
 	endstate = X_;
-	input = stdin;
 	sts = states;
 	cnt = 0;
 	col = 0;
@@ -2281,7 +2399,7 @@ mvdata(out,msg)
 	/*	  ... that only "Subject:" has 8-bit chars ...	     */
 	mail_priority = _MAILPRIO_NORMAL;
 	for (;;) {
-		c = getc(input);
+		c = s_getc();
 		/* An EOF in here is an error! */
 #if EOF != -1
 		if (c == EOF) return EOF;
@@ -2403,7 +2521,7 @@ mvdata(out,msg)
 
 	/* ================ Normal email BODY input.. ================ */
 	for (;;) {
-		c = getc(input);
+		c = s_getc();
 #if EOF != -1
 		if (c == EOF)		/* a little slower... */
 			return EOF;
@@ -2428,6 +2546,7 @@ mvdata(out,msg)
 		if (!ferror(out))
 			putc(c, out);
 	}
+	fflush(stdout);
 	return cnt;
 }
 
