@@ -106,6 +106,8 @@ ctlclose(dp)
 #ifdef	USE_MMAP
 	if (dp->let_buffer != NULL)
 	  munmap(dp->let_buffer, dp->let_end - dp->let_buffer);
+	if (dp->ctlmap != NULL)
+	  munmap(dp->ctlmap, dp->contentsize);
 #endif
 	if (dp->ctlfd >= 0)
 	  close(dp->ctlfd);
@@ -250,6 +252,23 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	  warning("Out of virtual memory!", (char *)NULL);
 	  exit(EX_SOFTWARE);
 	}
+#ifdef USE_MMAP
+#ifndef MAP_VARIABLE
+# define MAP_VARIABLE 0
+#endif
+#ifndef MAP_FILE
+# define MAP_FILE 0
+#endif
+	/* We do recipient locking via MMAP_SHARED RD/WR ! Less syscalls.. */
+	d.ctlmap = (char *)mmap(NULL, stbuf.st_size,
+				PROT_READ|PROT_WRITE,
+				MAP_FILE|MAP_SHARED|MAP_VARIABLE,
+				d.ctlfd, 0);
+	if ((int)d.ctlmap == (int)-1L)
+	  d.ctlmap = NULL; /* Failed ?? */
+#else
+	d.ctlmap = NULL;
+#endif
 	d.contentsize = stbuf.st_size;
 	d.contents[d.contentsize] = 0; /* Treat it as a long string.. */
 	if (read(d.ctlfd, d.contents, d.contentsize) != d.contentsize) {
@@ -269,6 +288,10 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	  free(d.contents);
 	  free((char *)d.offset);
 	  close(d.ctlfd);
+#ifdef USE_MMAP
+	  if (d.ctlmap != NULL)
+	    munmap(d.ctlmap, d.contentsize);
+#endif	  
 	  warning("Truncated or illegal control file \"%s\"!", file);
 	  /* exit(EX_PROTOCOL); */
 	  sleep(60);
@@ -336,7 +359,7 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 		    && !(*selectaddr)(host, ap->host))
 		|| (ap->routermxes == NULL && selectaddr == NULL
 		    && host != NULL && cistrcmp(host,ap->host) !=0)
-		|| !lockaddr(d.ctlfd, d.offset[i]+1,
+		|| !lockaddr(d.ctlfd, d.ctlmap, d.offset[i]+1,
 			     _CFTAG_NORMAL, _CFTAG_LOCK)) {
 	      if (ap->routermxes)
 		free((char *)ap->routermxes);
@@ -347,7 +370,7 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	    ap->link = d.senders; /* point at sender address */
 	    rp = (struct rcpt *)emalloc(sizeof (struct rcpt));
 	    if (rp == NULL) {
-	      lockaddr(d.ctlfd, d.offset[i]+1,
+	      lockaddr(d.ctlfd, d.ctlmap, d.offset[i]+1,
 		       _CFTAG_LOCK, _CFTAG_DEFER);
 	      warning("Out of virtual memory!", (char *)NULL);
 	      *exitflagp = 1;
@@ -537,9 +560,10 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	  return NULL;
 	}
 #ifdef	USE_MMAP
-	d.let_buffer = (char *)mmap(NULL, stbuf.st_size,
-				    PROT_READ, MAP_SHARED, d.msgfd, 0);
-	if ((long)d.let_buffer == -1L) {
+	d.let_buffer = (char *)mmap(NULL, stbuf.st_size, PROT_READ,
+				    MAP_FILE|MAP_SHARED|MAP_VARIABLE,
+				    d.msgfd, 0);
+	if ((int)d.let_buffer == (int)-1L) {
 	  warning("Out of MMAP() memory! Tried to map in (r/o) %d bytes (%m)",
 		  stbuf.st_size);
 	  free(mfpath);
