@@ -55,7 +55,10 @@ notary_setwtt(host)
 char *host;
 {
 	if (arg5) free(arg5);
-	arg5 = strdup(host);
+	if (host)
+	  arg5 = strdup(host);
+	else
+	  arg5 = NULL;
 }
 
 void
@@ -98,6 +101,7 @@ diagnostic(rp, rc, timeout, fmt, va_alist) /* (rp, rc, timeout, "fmtstr", remote
 	char	mark, message[8192];
 	register char *cp, *s, *es, *s2;
 	va_list	ap;
+	int report_notary = 1;
 
 	rp->status = rc;
 
@@ -137,8 +141,14 @@ diagnostic(rp, rc, timeout, fmt, va_alist) /* (rp, rc, timeout, "fmtstr", remote
 
 	retryat_time = 0;
 
+	/* XX: Following needs changes if we are to do positive
+	   delivery acknowledgements! */
+	if (rp->status == EX_OK)
+	  report_notary = 0;
+
 	printf("%d/%ld\t%s\t",
-	       rp->desc->ctlid, rp->id, notarybuf ? notarybuf : "");
+	       rp->desc->ctlid, rp->id,
+	       (notarybuf && report_notary) ? notarybuf : "");
 	switch (rp->status) {
 	case EX_OK:
 		printf("ok");
@@ -180,7 +190,8 @@ diagnostic(rp, rc, timeout, fmt, va_alist) /* (rp, rc, timeout, "fmtstr", remote
 	if (message[0] != '\0')
 		printf(" %s", message);
 	printf("\n");
-	if (!lockaddr(rp->desc->ctlfd, rp->lockoffset, _CFTAG_LOCK, mark)) {
+	if (!lockaddr(rp->desc->ctlfd, rp->desc->ctlmap,
+		      rp->lockoffset, _CFTAG_LOCK, mark)) {
 	  /* something went wrong in unlocking it, concurrency problem? */
 	}
 	rp->lockoffset = 0;	/* mark this recipient unlocked */
