@@ -420,19 +420,22 @@ main(argc, argv)
 	  currenteuid = 0;
 
 	  dp = ctlopen(file, channel, host, &getout, NULL, NULL);
-	  if (dp != NULL) {
-	    if (verboselog) {
-	      fclose(verboselog);
-	      verboselog = NULL;
-	    }
-	    if (dp->verbose) {
-	      verboselog = (FILE*)fopen(dp->verbose,"a");
-	      if (verboselog)
-		setbuf(verboselog,NULL);
-	    }
-	    process(dp);
-	    ctlclose(dp);
+	  if (dp == NULL) {
+	    printf("#resync %s\n",file);
+	    fflush(stdout);
+	    continue;
 	  }
+	  if (verboselog) {
+	    fclose(verboselog);
+	    verboselog = NULL;
+	  }
+	  if (dp->verbose) {
+	    verboselog = (FILE*)fopen(dp->verbose,"a");
+	    if (verboselog)
+	      setbuf(verboselog,NULL);
+	  }
+	  process(dp);
+	  ctlclose(dp);
 
 #if	defined(BIFF) || defined(RBIFF)
 	  /* now that the delivery phase is over, hoot'n wave */
